@@ -11,7 +11,10 @@
 # Author: John Egan <jwegan@gmail.com>
 
 import collections
-import ConfigParser
+try:
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
 import os.path
 
 FlawlessOption = collections.namedtuple("FlawlessOption", ["name", "default", "type", "description"])
@@ -38,6 +41,8 @@ OPTIONS = [
     FlawlessOption("smtp_host", "localhost:25", str, "The host responsbile for sending email"),
     FlawlessOption("smtp_user", None, str, "Username for smtp server (if required)"),
     FlawlessOption("smtp_password", None, str, "Password for smtp server (if required)"),
+    FlawlessOption("smtp_use_tls", False, bool, "Use TLs for smtp server"),
+    FlawlessOption("smtp_from", None, str, "From address to send emails from"),
 
     # Directory paths
     FlawlessOption("data_dir_path", "/tmp/", str, "Path to where flawless can store state to disk"),
@@ -75,6 +80,7 @@ OPTIONS = [
     # Logging
     FlawlessOption("log_level", "INFO", str, "The logging level"),
     FlawlessOption("log_file", None, str, "Path to the log file"),
+    FlawlessOption("log_days_to_keep", 3, int, "Number of days of log files to keep around"),
 
     # Storage
     FlawlessOption("redis_version", "2.0", str, "The version of the redis server if using redis"),
@@ -103,7 +109,7 @@ def get():
 
 def init_config(filepath):
     setattr(RUNTIME_CONFIG, "config_dir_path", os.path.dirname(filepath))
-    parser = ConfigParser.SafeConfigParser()
+    parser = configparser.SafeConfigParser()
     parser.read(filepath)
     for option in OPTIONS:
         if parser.has_option("flawless", option.name):
