@@ -14,11 +14,13 @@ from __future__ import absolute_import
 import __main__
 
 import cgi
-import copy
-import email
+try:
+    from email.mime.text import MIMEText
+except ImportError:
+    from email.MIMEText import MIMEText
 import collections
 import datetime
-import inspect
+from functools import reduce
 import logging
 import os
 import os.path
@@ -30,11 +32,17 @@ import threading
 import time
 import traceback
 import urllib
-import urlparse
+try:
+    import urllib.parse
+    urlencode = urllib.parse.urlencode
+except ImportError:
+    import urllib
+    urlencode = urllib.urlencode
 
-from thrift.Thrift import TType
+from future.utils import iteritems
 
 import flawless.lib.config
+from flawless.lib.data_structures.lru_cache import ExpiringLRUCache
 from flawless.lib.data_structures import prefix_tree
 from flawless.lib.storage import DiskStorage
 from flawless.lib.version_control.repo import get_repository
@@ -71,7 +79,7 @@ def code_identifier_equality(self, other):
 
 api_ttypes.CodeIdentifier.__eq__ = code_identifier_equality
 api_ttypes.KnownError.__eq__ = code_identifier_equality
-api_ttypes.ErrorKey.__hash__ = lambda self: reduce(lambda x, y: x ^ hash(y), self.__dict__.iteritems(), 1)
+api_ttypes.ErrorKey.__hash__ = lambda self: reduce(lambda x, y: x ^ hash(y), iteritems(self.__dict__), 1)
 
 
 ############################## BASE SERVICE CLASS ##############################
@@ -85,7 +93,7 @@ class FlawlessServiceBaseClass(object):
                  open_process_func=subprocess.Popen,
                  smtp_client_cls=smtplib.SMTP,
                  time_func=time.time):
-        self.__dict__.update({k: v for k, v in locals().iteritems() if k != 'self'})
+        self.__dict__.update({k: v for k, v in iteritems(locals()) if k != 'self'})
         self.extract_base_path_patterns = [re.compile('^.*/%s/?(.*)$' % d) for d in
                                            config.report_runtime_package_directory_names]
         self.raise_exception_pattern = re.compile('^\s*raise[ \n]')
@@ -105,10 +113,14 @@ class FlawlessServiceBaseClass(object):
                                                            api_ttypes.CodeIdentifierList())
         self.known_errors = self._parse_whitelist(config_storage["known_errors"] or
                                                   api_ttypes.KnownErrorList())
+        self.ignored_exceptions = set((config_storage["ignored_exceptions"] or
+                                       api_ttypes.IgnoredExceptionList()).exceptions)
 
         # Watchers
         self.watch_all_errors, self.watch_only_if_blamed = self._parse_watchers_file(config_storage["watch_list"] or
                                                                                      api_ttypes.WatchList())
+        self.disownerships = self._parse_disownership(config_storage["disownership_list"] or
+                                                      api_ttypes.FileDisownershipList())
 
         # Email remapping
         self.email_remapping = config_storage["email_remapping"] or api_ttypes.EmailRemapping()
@@ -140,6 +152,22 @@ class FlawlessServiceBaseClass(object):
 
         return all_error_tree, blame_only_tree
 
+    def _parse_disownership(self, disownership_list):
+        disownership_tree = prefix_tree.FilePathTree()
+        for entry in disownership_list.disownerships:
+            if entry.filepath not in disownership_tree:
+                disownership_tree[entry.filepath] = list()
+            disownership_tree[entry.filepath].append(entry)
+
+        # Set disownership_tree to have accumulator that will allow us to find everyone who has disowned
+        # the file or a parent of the file
+        disownership_tree.set_accumulator(
+            accumulator_intializer=list(),
+            accumulator_func=lambda x, y: x + y if y else x,
+        )
+        return disownership_tree
+
+
     ############################## Timestamp Helpers ##############################
 
     def _partition_for_ms(self, epoch_ms):
@@ -214,11 +242,24 @@ class FlawlessServiceBaseClass(object):
                 return match.group(1)
         return None
 
-    def _format_traceback(self, request, append_locals=True, show_full_stack=False,
+    def _format_traceback(self, err_info, append_locals=True, show_full_stack=False, include_err_info=False,
                           linebreak="<br />", spacer="&nbsp;", start_bold="<strong>",
                           end_bold="</strong>", escape_func=cgi.escape):
-        # Traceback
+        request = err_info.last_error_data
         parts = []
+        if include_err_info:
+            parts.append("{b}Current Exception Count:{xb} {count}{lb}".format(
+                b=start_bold, xb=end_bold, count=err_info.error_count, lb=linebreak))
+            parts.append("{b}Last Occurred:{xb} {last_occurrence}{lb}".format(
+                b=start_bold, xb=end_bold, last_occurrence=err_info.last_occurrence, lb=linebreak))
+            parts.append("{b}Last Occurrence Logged From:{xb} {hostname}{lb}".format(
+                b=start_bold, xb=end_bold, hostname=request.hostname, lb=linebreak))
+
+        if request.exception_type:
+            parts.append("{b}Exception Type:{xb} {type}{lb}".format(
+                b=start_bold, xb=end_bold, type=request.exception_type, lb=linebreak))
+
+        # Traceback
         parts.append("{b}Traceback (most recent call last):{xb}".format(b=start_bold, xb=end_bold))
         formatted_stack = [
             '{sp}{sp}File "{filename}", line {line}, in {function}{lb}{sp}{sp}{sp}{sp}{code}'.format(
@@ -230,6 +271,11 @@ class FlawlessServiceBaseClass(object):
         parts.extend(formatted_stack)
         parts.append(escape_func(request.exception_message))
 
+        def decode(value):
+            if hasattr(value, 'decode'):
+                return value.decode('UTF-8', 'replace')
+            return value
+
         # Frame Locals
         parts.append(linebreak * 2 + "{b}Stack Frame:{xb}".format(b=start_bold, xb=end_bold))
         types_to_show = [api_ttypes.LineType.KNOWN_ERROR,
@@ -243,7 +289,7 @@ class FlawlessServiceBaseClass(object):
                 b=start_bold, xb=end_bold,
             )
             local_vals = ['{sp}{sp}{sp}{sp}{name}={value}'.format(
-                          sp=spacer, name=name, value=escape_func(value.decode("UTF-8", "replace")))
+                          sp=spacer, name=name, value=escape_func(decode(value)))
                           for name, value in sorted(l.frame_locals.items())]
             parts.append(line_info)
             parts.extend(local_vals or [spacer * 4 + "No variables in this frame"])
@@ -252,7 +298,7 @@ class FlawlessServiceBaseClass(object):
         if request.additional_info:
             parts.append(linebreak * 2 + "{b}Additional Information:{xb}".format(b=start_bold, xb=end_bold))
             parts.append(
-                escape_func(request.additional_info.decode("UTF-8", "replace")).replace("\n", linebreak)
+                escape_func(decode(request.additional_info)).replace("\n", linebreak)
             )
 
         return linebreak.join(parts)
@@ -280,6 +326,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         self.number_of_git_blames_running = 0
 
         self.repository = get_repository(open_process_func=self.open_process_func)
+        self.emailed_errors_cache = ExpiringLRUCache(size=1000, expiration_seconds=60 * 5)
 
         self.lock = threading.RLock()
         self.errors_seen = None
@@ -329,9 +376,6 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         return True
 
     def _sendmail(self, to_addresses, subject, body):
-        host, port = config.smtp_host.split(":")
-        smtp_client = self.smtp_client_cls(host, int(port))
-
         invalid_addresses = [e for e in to_addresses if not bool(self.VALIDATE_EMAIL_PATTERN.match(e))]
         if invalid_addresses:
             to_addresses = [e for e in to_addresses if e not in invalid_addresses]
@@ -340,13 +384,19 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
                 log_func=log.warning,
             )
 
-        msg = email.MIMEText.MIMEText(body.encode("UTF-8"), "html", "UTF-8")
-        msg["From"] = "flawless@%s" % config.email_domain_name
+        msg = MIMEText(body.encode("UTF-8"), "html", "UTF-8")
+        msg["From"] = config.smtp_from or "flawless@%s" % config.email_domain_name
         msg["To"] = ", ".join(to_addresses)
         msg["Subject"] = subject
 
+        host, port = config.smtp_host.split(":")
+        smtp_client = self.smtp_client_cls(host, int(port))
+
+        if config.smtp_use_tls:
+            smtp_client.starttls()
         if config.smtp_user and config.smtp_password:
             smtp_client.login(config.smtp_user, config.smtp_password)
+
         smtp_client.sendmail(msg["From"], to_addresses, msg.as_string())
         smtp_client.quit()
 
@@ -386,12 +436,17 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
     def _record_error(self, request):
         log.debug("Recieved error from %s for %s" % (request.hostname, request.exception_message))
 
+        # Skip ignored exceptions (ex: connection errors)
+        if request.exception_type and request.exception_type in self.ignored_exceptions:
+            return
+
         # Figure out which line in the stack trace is to blame for the error
         key, blamed_entry, email_recipients, was_whitelisted = self._blame_line(request.traceback)
         if not key:
             if not was_whitelisted:
                 name_map = api_ttypes.LineType._VALUES_TO_NAMES
-                line_types = [(stack_line.filename, name_map.get(self._get_line_type(stack_line)))
+                line_types = [("%s:%d" % (stack_line.filename, stack_line.line_number),
+                               name_map.get(self._get_line_type(stack_line)))
                               for stack_line in request.traceback]
                 log.info("Unable to blame: %s" % str(line_types))
             return
@@ -399,7 +454,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         # If this error hasn't been reported before, then find the dev responsible
         err_info = None
         if key not in self.errors_seen:
-            # If flawless is being flooded wih errors, limit the number of git blames so the
+            # If flawless is being flooded with errors, limit the number of git blames so the
             # service doesn't fall over. We don't use a thread safe counter, because 10
             # git blames is just a soft limit
             if self.number_of_git_blames_running > config.max_concurrent_git_blames:
@@ -411,9 +466,13 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
                 email, last_touched_ts = self.repository.blame(key.filename, key.line_number)
             finally:
                 self.number_of_git_blames_running -= 1
-            dev_email = self._get_email(email)
-            last_touched_ts = last_touched_ts or 0
 
+            dev_email = self._get_email(email)
+            if key.filename in self.disownerships:
+                remapped_owners = {entry.email: entry.designated_email for entry in self.disownerships[key.filename]}
+                dev_email = remapped_owners.get(dev_email, dev_email)
+
+            last_touched_ts = last_touched_ts or 0
             cur_time = self._convert_epoch_ms(datetime.datetime).strftime("%Y-%m-%d %H:%M:%S")
             mod_time = self._convert_epoch_ms(datetime.datetime, epoch_ms=last_touched_ts * 1000)
             mod_time = mod_time.strftime("%Y-%m-%d %H:%M:%S")
@@ -437,7 +496,7 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         # If we've already seen this error then update the error count
         elif key in self.errors_seen:
             err_info = self.errors_seen[key]
-            err_info.error_count += 1
+            err_info.error_count += request.error_count or 1
             err_info.last_error_data = request
             cur_dt = self._convert_epoch_ms(datetime.datetime)
             err_info.last_occurrence = cur_dt.strftime("%Y-%m-%d %H:%M:%S")
@@ -457,14 +516,14 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
         else:
             # If it is a known error, we allow fine grainted control of how frequently emails will
             # be sent. An email will be sent if it has passed the min_alert_threshold, and/or this
-            # is the Nth occurrence as defined alert_every_n_occurences. If it has passed
+            # is the Nth occurrence as defined alert_every_n_occurrences. If it has passed
             # max_alert_threshold then no emails will be sent.
             known_entry = self._get_entry(blamed_entry, self.known_errors)
             if (known_entry.min_alert_threshold and err_info.error_count >= known_entry.min_alert_threshold
                     and not err_info.email_sent):
                 send_email = True
-            if (known_entry.alert_every_n_occurences and
-                    err_info.error_count % known_entry.alert_every_n_occurences == 0):
+            if (known_entry.alert_every_n_occurrences and
+                    err_info.error_count % known_entry.alert_every_n_occurrences == 0):
                 send_email = True
             if (known_entry.max_alert_threshold is not None
                     and err_info.error_count > known_entry.max_alert_threshold):
@@ -487,332 +546,42 @@ class FlawlessThriftServiceHandler(FlawlessServiceBaseClass):
             email_recipients.extend(known_entry.email_recipients or [])
             email_body.append(known_entry.email_header or "")
 
-        email_body.append(self._format_traceback(request))
+        email_body.append(self._format_traceback(err_info))
         email_body.append(
-            "<br /><br /><a href='http://%s/add_known_error?%s'>Add to whitelist</a>" %
+            "<br /><br /><a href='%s/view_traceback?%s'>View most recent occurrence</a>" %
             (
                 config.hostname,
-                urllib.urlencode(
+                urlencode(
+                    dict(filename=err_key.filename, function_name=err_key.function_name, text=err_key.text,
+                         line_number=err_key.line_number, timestamp=self._epoch_ms() / 1000)
+                )
+            )
+        )
+        email_body.append("<br /><a href='%s#%s'>View all errors assigned to you</a>" % (config.hostname, dev_email))
+        email_body.append(
+            "<br /><a href='%s/add_known_error?%s'>Add to whitelist</a>" %
+            (
+                config.hostname,
+                urlencode(
                     dict(filename=err_key.filename, function_name=err_key.function_name, code_fragment=err_key.text)
                 )
             )
         )
+        email_body.append("<br /><a href='https://github.com/shopkick/flawless'>What is Flawless?</a>")
+
+        if err_key in self.emailed_errors_cache:
+            return  # Skip sending this email if we've already emailed it in the past 5 minutes
+        self.emailed_errors_cache[err_key] = True
 
         # Send the email
         log.info("Sending email for %s to %s" % (str(err_key), ", ".join(email_recipients)))
-        self._sendmail(
-            to_addresses=email_recipients,
-            subject="Error on %s in %s" % (request.hostname, err_key.filename),
-            body="<br />".join([s for s in email_body if s]),
-        )
-        err_info.email_sent = True
-        self.errors_seen[err_key] = err_info
-
-
-############################## WEB SERVICE ##############################
-
-class FlawlessWebServiceHandler(FlawlessServiceBaseClass):
-    """Handler for HTTP server to show state of the Flawless service"""
-
-    def __init__(self, *args, **kwargs):
-        super(FlawlessWebServiceHandler, self).__init__(*args, **kwargs)
-
-    def index(self, *args, **kwargs):
-        return self.get_weekly_error_report(*args, **kwargs)
-
-    def _get_errors_seen_for_ts(self, timestamp):
-        prefix = self._partition_for_ms(int(timestamp) * 1000 if timestamp else None)
-        errors_seen = self.storage_factory(prefix)
-        errors_seen.open()
-        return errors_seen
-
-    def _add_new_entry_to_config(self, key, entry, attr='identifiers'):
-        class_map = dict(known_errors=api_ttypes.KnownErrorList,
-                         building_blocks=api_ttypes.CodeIdentifierList,
-                         third_party_whitelist=api_ttypes.CodeIdentifierList,
-                         watch_list=api_ttypes.WatchList)
-        config_storage = self.storage_factory(partition=None)
-        config_storage.open()
-        current_value = config_storage[key] or class_map[key]()
-        getattr(current_value, attr).append(entry)
-        current_value.last_update_ts = self._epoch_ms()
-        config_storage[key] = current_value
-        config_storage.sync()
-        config_storage.close()
-
-    def _construct_instance(self, params, cls):
-        THRIFT_SPEC_NAME_FIELD = 2
-        THRIFT_SPEC_TYPE_FIELD = 1
-        init_args = inspect.getargspec(cls.__init__).args
-        whitelist_attrs = [s for s in init_args if s != 'self']
-        args = dict((k, params.get(k)) for k in whitelist_attrs)
-        arg_type_map = dict()
-
-        # Build map of fields to type based on the thrift spec
-        for spec in cls.thrift_spec:
-            if not spec or len(spec) < 3:
-                continue
-            if spec[THRIFT_SPEC_TYPE_FIELD] == TType.BOOL:
-                arg_type_map[spec[THRIFT_SPEC_NAME_FIELD]] = bool
-            elif spec[THRIFT_SPEC_TYPE_FIELD] == TType.I64 or spec[THRIFT_SPEC_TYPE_FIELD] == TType.I32:
-                arg_type_map[spec[THRIFT_SPEC_NAME_FIELD]] = int
-
-        # Cast args to the appropriate type
-        for field, value in args.items():
-            if value and field in arg_type_map:
-                args[field] = arg_type_map[field](value)
-
-        new_entry = cls(**args)
-        return new_entry
-
-    def get_weekly_error_report(self, timestamp=None, include_known_errors=False,
-                                include_modified_before_min_date=False):
-        errors_seen = self._get_errors_seen_for_ts(timestamp)
-        html_parts = ["<html><head><title>Error Report</title></head><body>"]
-
-        grouped_errors = collections.defaultdict(list)
-        developer_score = collections.defaultdict(int)
-        for key, value in errors_seen.iteritems():
-            if ((not value.is_known_error or include_known_errors) and
-                    (value.date >= config.report_only_after_minimum_date or include_modified_before_min_date)):
-                grouped_errors[value.developer_email].append((key, value))
-                developer_score[value.developer_email] += value.error_count
-
-        for developer, score in sorted(developer_score.items(), key=lambda t: t[1], reverse=True):
-            html_parts.append("<strong>%s (score: %d)</strong>" % (developer, score))
-            for err_key, err_info in grouped_errors[developer]:
-                html_parts.append("Number of Occurrences: " + str(err_info.error_count))
-                html_parts.append("Last Occurred: " + err_info.last_occurrence)
-                html_parts.append("Filename: " + err_key.filename)
-                html_parts.append("Function Name: " + err_key.function_name)
-                html_parts.append("Line Number: " + str(err_key.line_number))
-                html_parts.append("Date Committed: " + err_info.date)
-                html_parts.append("Email Sent: " + str(err_info.email_sent))
-                params = copy.copy(err_key.__dict__)
-                if timestamp:
-                    params["timestamp"] = timestamp
-                view_url = "%s/view_traceback?%s" % (config.hostname, urllib.urlencode(params))
-                html_parts.append("<a href='%s'>view traceback</a>" % view_url)
-                html_parts.append("<br />")
-            html_parts.append("<br />")
-
-        if not grouped_errors:
-            html_parts.append("Wow, no errors. Great job!")
-        return "<br />".join(html_parts)
-
-    def view_traceback(self, filename="", function_name="", text="", line_number="", timestamp=None):
-        errors_seen = self._get_errors_seen_for_ts(timestamp)
-        err_key = api_ttypes.ErrorKey(filename=filename, function_name=function_name,
-                                      text=text, line_number=int(line_number))
-
-        err_info = errors_seen[err_key]
-        if err_info:
-            datastr = self._format_traceback(err_info.last_error_data)
-        else:
-            datastr = "Not Found"
-
-        return """
-            <html>
-                <head>
-                    <title>Flawless Traceback</title>
-                </head>
-                <body style='font-family: courier; font-size: 10pt'>
-                    {data}
-                </body>
-            </html>
-        """.format(data=datastr)
-
-    ############################## Add New Known Error ##############################
-
-    def add_known_error(self, filename="", function_name="", code_fragment=""):
-        code_fragment = cgi.escape(code_fragment)
-        return """
-            <html>
-                <head>
-                    <title>Add Known Error</title>
-                </head>
-                <body>
-                <div>
-                    Instructions: Fill out the file path, function name and code fragment for the known error.
-                    If function name or code fragment are left empty, then they are treated as wildcards.<br />
-                    Just entering file path, function name and code fragment will whitelist the error and stop
-                    all emails about it. If you want to continue emails, but at a lower (or higher)
-                    frequency or threshold you can use the optional fields.
-                </div><br /><br />
-                <form action='save_known_error' method='POST'>
-                    <table>
-                        <tr><td>* = Required</td></tr>
-                        <tr>
-                            <td>* File Path:</td>
-                            <td><input name='filename' type='text' value='{filename}' size='50'/></td>
-                        </tr>
-                        <tr>
-                            <td>* Function Name:</td>
-                            <td><input name='function_name' type='text'value='{function_name}' size='50'/></td>
-                        </tr>
-                        <tr>
-                            <td>* Code Fragment:</td>
-                            <td><textarea name='code_fragment' rows='1' cols='50'/>{code_fragment}</textarea></td>
-                        </tr>
-                        <tr>
-                            <td>* Error Type:</td>
-                            <td>
-                                <select name='type'>
-                                    <option value='known_errors' selected>Add to Known Errors</option>
-                                    <option value='building_blocks'>Mark as Library Code</option>
-                                    <option value='third_party_whitelist'>Add to Ignored Thirdparty Errors</option>
-                                </select>
-                            </td>
-                        </tr>
-                        <tr><td>&nbsp</td></tr>
-                        <tr><td><strong>Following section is only for known errors</strong></td></tr>
-                        <tr><td>Must set one of the following **</td></tr>
-                        <tr>
-                            <td>** Minimum Alert Threshold:</td>
-                            <td><input name='min_alert_threshold' type='text' /></td>
-                        </tr>
-                        <tr>
-                            <td>** Maximum Alert Threshold:</td>
-                            <td><input name='max_alert_threshold' type='text' /></td>
-                        </tr>
-                        <tr>
-                            <td>** Alert Every N Occurrences:</td>
-                            <td><input name='alert_every_n_occurences' type='text' /></td>
-                        </tr>
-                        <tr>
-                            <td>Email Recipients CSV:</td>
-                            <td><input name='email_recipients' type='text' size='50'/></td>
-                        </tr>
-                        <tr>
-                            <td>Email Header:</td>
-                            <td><textarea name='email_header' rows='5' cols='50'></textarea></td>
-                        </tr>
-                    </table>
-                    <input type='submit'></input>
-                </form>
-             </body>
-            </html>
-        """.format(**dict(locals().items()))
-
-    def save_known_error(self, request):
-        params = dict(urlparse.parse_qsl(request))
-        class_map = dict(known_errors=api_ttypes.KnownError, building_blocks=api_ttypes.CodeIdentifier,
-                         third_party_whitelist=api_ttypes.CodeIdentifier)
-        new_entry = self._construct_instance(params, class_map[params['type']])
-        self._add_new_entry_to_config(params['type'], new_entry)
-
-        return "<html><body>SUCCESS</body></html>"
-
-    def add_watch(self):
-        return """
-            <html>
-                <head>
-                    <title>Add Watch</title>
-                </head>
-                <body>
-                <div>
-                    Instructions: Fill out the file path & email to send reports to.
-                </div><br /><br />
-                <form action='save_watch' method='POST'>
-                    <table>
-                        <tr><td>* = Required</td></tr>
-                        <tr>
-                            <td>* File Path:</td>
-                            <td><input name='filepath' type='text' size='50'/></td>
-                        </tr>
-                        <tr>
-                            <td>* Email:</td>
-                            <td><input name='email' type='text' size='50'/></td>
-                        </tr>
-                        <tr>
-                            <td>* Watch Type:</td>
-                            <td>
-                                <select name='watch_all_errors'>
-                                    <option value='true' selected>Any Error</option>
-                                    <option value='false'>Only Blamed Errors</option>
-                                </select>
-                            </td>
-                        </tr>
-                    </table>
-                    <input type='submit'></input>
-                </form>
-             </body>
-            </html>
-        """
-
-    def save_watch(self, request):
-        params = dict(urlparse.parse_qsl(request))
-        params['watch_all_errors'] = params['watch_all_errors'] == 'true'
-        new_entry = self._construct_instance(params, api_ttypes.WatchFileEntry)
-        self._add_new_entry_to_config("watch_list", new_entry, attr="watches")
-        return "<html><body>SUCCESS</body></html>"
-
-    def remap_email(self):
-        return """
-            <html>
-                <head>
-                    <title>Remap Email</title>
-                </head>
-                <body>
-                <div>
-                    Instructions: Fill out the old email & the new email to send reports to.
-                </div><br /><br />
-                <form action='save_remap_email' method='POST'>
-                    <table>
-                        <tr><td>* = Required</td></tr>
-                        <tr>
-                            <td>* Old Email:</td>
-                            <td><input name='old_email' type='text' size='50'/></td>
-                        </tr>
-                        <tr>
-                            <td>* New Email:</td>
-                            <td><input name='new_email' type='text' size='50'/></td>
-                        </tr>
-                    </table>
-                    <input type='submit'></input>
-                </form>
-             </body>
-            </html>
-        """
-
-    def save_remap_email(self, request):
-        params = dict(urlparse.parse_qsl(request))
-        config_storage = self.storage_factory(partition=None)
-        config_storage.open()
-        current_value = config_storage["email_remapping"] or api_ttypes.EmailRemapping()
-        current_value.remap[params["old_email"]] = params["new_email"]
-        current_value.last_update_ts = self._epoch_ms()
-        config_storage["email_remapping"] = current_value
-        config_storage.sync()
-        config_storage.close()
-        return "<html><body>SUCCESS</body></html>"
-
-    def view_config(self, key):
-        config_storage = self.storage_factory(partition=None)
-        config_storage.open()
-        current_value = config_storage[key]
-        config_storage.close()
-
-        if pprint:
-            data = pprint.pformat(current_value)
-        else:
-            data = str(current_value)
-
-        data = data.replace("\n", "<br />")
-        return """
-            <html>
-                <head>
-                    <title>Flawless Config</title>
-                </head>
-                <body style='font-family: courier; font-size: 10pt'>
-                    {data}
-                </body>
-            </html>
-        """.format(data=data)
-
-    def check_health(self):
-        parts = ["<html><body>OK<br/>"]
-        for option in flawless.lib.config.OPTIONS:
-            parts.append("%s: %s" % (option.name, str(getattr(config, option.name))))
-        parts.append("</body></html>")
-        return "<br />".join(parts)
+        try:
+            self._sendmail(
+                to_addresses=email_recipients,
+                subject="Error on %s in %s" % (request.hostname, err_key.filename),
+                body="<br />".join([s for s in email_body if s]),
+            )
+            err_info.email_sent = True
+            self.errors_seen[err_key] = err_info
+        except smtplib.SMTPRecipientsRefused:
+            log.error("Unable to send email to: %s for %s" % (str(err_key), ", ".join(email_recipients)))
