@@ -15,7 +15,10 @@ import datetime
 import email
 import time
 import unittest
-import StringIO
+try:
+    from StringIO import StringIO
+except ImportError:
+    from io import StringIO
 
 from flawless.lib.storage.stub import StubStorage
 import flawless.lib.config
@@ -29,7 +32,7 @@ class BaseTestCase(unittest.TestCase):
     def setUp(self):
         super(BaseTestCase, self).setUp()
         self.popen_stub = POpenStub()
-        self.popen_stub.stdout = StringIO.StringIO(
+        self.popen_stub.stdout = StringIO(
             "75563df6e9d1efe44b48f6643fde9ebbd822b0c5 25 25 1\n"
             "author John Egan\n"
             "author-mail <wishbone@shopkick.com>\n"
@@ -93,7 +96,7 @@ class BaseTestCase(unittest.TestCase):
                               'api_ttypes.WaterbowlErrorCode.OUT_OF_WATER, message=str(e))',
                 email_recipients=["wilfred@shopkick.com", "snoopy@shopkick.com"],
                 email_header='NOTE: This error typically does not require dev team involvement.',
-                alert_every_n_occurences=1,
+                alert_every_n_occurrences=1,
             ),
         ])
 
@@ -103,11 +106,23 @@ class BaseTestCase(unittest.TestCase):
                                       'raise errors.BadRequestError("Missing param %s" % name)'),
         ])
 
+        self.ignored_exceptions = api_ttypes.IgnoredExceptionList(['exceptions.BananaException'])
+
+        self.disowned_files = api_ttypes.FileDisownershipList([
+            api_ttypes.FileDisownershipEntry(
+                email='wishbone@shopkick.com',
+                filepath='scripts',
+                designated_email='lassie@shopkick.com'
+            )
+        ])
+
         self.config_storage_stub = StubStorage(partition=None)
         self.config_storage_stub["watch_list"] = self.watchers
         self.config_storage_stub["third_party_whitelist"] = self.third_party_whitelist
         self.config_storage_stub["known_errors"] = self.known_errors
         self.config_storage_stub["building_blocks"] = self.building_blocks
+        self.config_storage_stub["ignored_exceptions"] = self.ignored_exceptions
+        self.config_storage_stub["disownership_list"] = self.disowned_files
         self.errors_storage_stub = StubStorage(partition=None)
 
         self.saved_config = copy.deepcopy(flawless.lib.config.get().__dict__)
@@ -120,6 +135,7 @@ class BaseTestCase(unittest.TestCase):
             r"^coreservices(?!.*/thrift/).*$",
             r"lib/.*",
             r"tools/.*",
+            r"scripts/.*"
         ]
         self.test_config.report_runtime_package_directory_names = ["site-packages"]
         self.test_config.config_dir_path = "../config"
@@ -156,13 +172,13 @@ class BaseTestCase(unittest.TestCase):
                                      (key, str(expected[key]), str(actual[key])))
 
     def assertEmailEquals(self, expected, actual):
-        self.assertEquals(expected["from_address"], actual["from_address"])
-        self.assertEquals(set(expected["to_addresses"]), set(actual["to_addresses"]))
+        self.assertEqual(expected["from_address"], actual["from_address"])
+        self.assertEqual(set(expected["to_addresses"]), set(actual["to_addresses"]))
         parsed_email = email.message_from_string(actual["body"])
-        self.assertEquals(expected["from_address"], parsed_email["From"])
-        self.assertEquals(set(expected["to_addresses"]), set(parsed_email["To"].split(", ")))
-        self.assertEquals(expected["subject"], parsed_email["Subject"])
-        self.assertTrue(expected["body"] in parsed_email.get_payload(decode=True))
+        self.assertEqual(expected["from_address"], parsed_email["From"])
+        self.assertEqual(set(expected["to_addresses"]), set(parsed_email["To"].split(", ")))
+        self.assertEqual(expected["subject"], parsed_email["Subject"])
+        self.assertTrue(expected["body"] in parsed_email.get_payload(decode=True).decode('utf8'))
 
 
 class RecordErrorTestCase(BaseTestCase):
@@ -221,6 +237,19 @@ class RecordErrorTestCase(BaseTestCase):
         self.handler.record_error(req)
         self.assertDictEquals({}, self.handler.errors_seen.dict)
 
+    def test_ignores_ignored_exceptions(self):
+        req = api_ttypes.RecordErrorRequest(
+            traceback=[api_ttypes.StackLine("/site-packages/lib/test.py", 5, "test_func", "code"),
+                       api_ttypes.StackLine("/site-packages/coreservices/service.py", 7, "serve", "..."),
+                       api_ttypes.StackLine("/site-packages/thirdparty/3rdparty_lib.py", 9, "call", "x")],
+            exception_message="email text",
+            exception_type="exceptions.BananaException",
+            hostname="localhost",
+        )
+
+        self.handler.record_error(req)
+        self.assertDictEquals({}, self.handler.errors_seen.dict)
+
     def test_doesnt_report_errors_under_threshold(self):
         self.test_config.report_error_threshold = 2
         req = api_ttypes.RecordErrorRequest(
@@ -237,7 +266,7 @@ class RecordErrorTestCase(BaseTestCase):
                 1, "wishbone@shopkick.com", "2017-07-30 00:00:00", False, "2020-01-01 00:00:00",
                 is_known_error=False, last_error_data=req)},
             self.handler.errors_seen.dict)
-        self.assertEquals(None, self.smtp_stub.last_args)
+        self.assertEqual(None, self.smtp_stub.last_args)
 
     def test_uses_threshold_specified_in_request(self):
         self.test_config.report_error_threshold = 2
@@ -344,7 +373,7 @@ class RecordErrorTestCase(BaseTestCase):
                                     body='NOTE: This error typically does not require dev team involvement.',
                                     smtp_server_host_port=None),
                                self.smtp_stub.last_args)
-        body = email.message_from_string(self.smtp_stub.last_args["body"]).get_payload(decode=True)
+        body = email.message_from_string(self.smtp_stub.last_args["body"]).get_payload(decode=True).decode('utf8')
         self.assertTrue("email text" in body)
         self.assertTrue("extra stuff" in body)
 
@@ -381,7 +410,7 @@ class RecordErrorTestCase(BaseTestCase):
             exception_message="email text",
             hostname="localhost",
         )
-        self.popen_stub.stdout = StringIO.StringIO(
+        self.popen_stub.stdout = StringIO(
             "75563df6e9d1efe44b48f6643fde9ebbd822b0c5 25 25 1\n"
             "author John Egan\n"
             "author-mail <wishbone@shopkick.com>\n"
@@ -434,7 +463,7 @@ class RecordErrorTestCase(BaseTestCase):
                                                         False, "2020-01-01 00:00:00", is_known_error=True,
                                                         last_error_data=req)},
                               self.handler.errors_seen.dict)
-        self.assertEquals(None, self.smtp_stub.last_args)
+        self.assertEqual(None, self.smtp_stub.last_args)
 
     def test_ignores_third_party_whitelisted_errors(self):
         req = api_ttypes.RecordErrorRequest(
@@ -482,6 +511,31 @@ class RecordErrorTestCase(BaseTestCase):
                 is_known_error=False, last_error_data=req)},
             self.handler.errors_seen.dict)
 
+    def test_handles_disowned_files(self):
+        req = api_ttypes.RecordErrorRequest(
+            traceback=[api_ttypes.StackLine("/site-packages/lib/test.py", 5, "test_func", "code"),
+                       api_ttypes.StackLine("/site-packages/scripts/my_script.py", 7, "run_all", "code"),
+                       api_ttypes.StackLine("/site-packages/thirdparty/3rdparty_lib.py", 9, "call", "x")],
+            exception_message="email text",
+            hostname="localhost",
+        )
+
+        self.handler.record_error(req)
+        self.assertDictEquals({
+            api_ttypes.ErrorKey("scripts/my_script.py", 7, "run_all", "code"): api_ttypes.ErrorInfo(
+                1, "lassie@shopkick.com", "2017-07-30 00:00:00", True, "2020-01-01 00:00:00",
+                is_known_error=False, last_error_data=req)},
+            self.handler.errors_seen.dict)
+        self.assertEqual(["git", "--git-dir=/tmp/.git", "--work-tree=/tmp", "blame",
+                          "-p", "/tmp/scripts/my_script.py", "-L", "7,+1"],
+                         self.popen_stub.last_args)
+        self.assertEmailEquals(dict(to_addresses=["lassie@shopkick.com"],
+                                    from_address="flawless@example.com",
+                                    subject="Error on localhost in scripts/my_script.py",
+                                    body="email text",
+                                    smtp_server_host_port=None),
+                               self.smtp_stub.last_args)
+
 
 ############################## Stubs ##############################
 class LogStub(object):
@@ -499,8 +553,8 @@ class POpenStub(object):
 
     def __init__(self):
         self.last_args = None
-        self.stdout = StringIO.StringIO()
-        self.stderr = StringIO.StringIO()
+        self.stdout = StringIO()
+        self.stderr = StringIO()
 
     def __call__(self, args, **kwargs):
         self.last_args = args
@@ -536,7 +590,7 @@ class OpenFileStub(object):
         self.current_file = None
 
     def set_file(self, filename, contents):
-        self.files[filename] = StringIO.StringIO(contents)
+        self.files[filename] = StringIO(contents)
 
     def __enter__(self, *args, **kwargs):
         return self.files[self.current_file]
